/**
 * READ  带 
 */

import java.io.IOException;
import java.nio.channels.Selector;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * @author lty
 *
 */
public class ReadSelectorPool
{
	private Queue	queue	= null;
	protected final List pool = new ArrayList();
	private int 	size = 0;
	
	private static ReadSelectorPool	instance	= null;

	public static ReadSelectorPool getInstance(Queue queue, int size)
	{
		if(instance == null)
		{
			synchronized (ReadSelectorPool.class)
			{
				instance = new ReadSelectorPool(queue, size);
			}
		}
		return instance;
	}	
	
	public static ReadSelectorPool getInstance(Queue queue)
	{
		if(instance == null)
		{
			synchronized (ReadSelectorPool.class)
			{
				instance = new ReadSelectorPool(queue);
			}
		}
		return instance;
	}	
	public ReadSelectorPool(Queue queue)
	{
		this(queue, 1);
	}

	public ReadSelectorPool(Queue queue, int size)
	{
		this.size = size;
		this.queue = queue;
		init();
	}

	private void init()
	{
		for(int i = 0; i < size; i++)
		{
			pool.add(createHandler(i));
		}
	}

	protected Thread createHandler(int index)
	{
		Selector selector = null;
		try
		{
			selector = Selector.open();
		}
		catch(IOException e)
		{
			e.printStackTrace();
		}
		Thread handler = new ReadSelectorMonitor(queue, selector, index);

		return handler;
	}
	
	public Thread getReadMonitor()
	{
		Iterator iter = pool.iterator();
		ReadSelectorMonitor minMonitor = null;
		int compare = 0;
		
		while(iter.hasNext())
		{
			ReadSelectorMonitor monitor = (ReadSelectorMonitor) iter.next();
			int s = monitor.getRegistSize();
			if(minMonitor == null)
				minMonitor = monitor;
			
			if(s <= compare)
			{
				minMonitor = monitor;
			}
			compare = s;
			
		}
		
		return minMonitor;
		
	}
	

	public void startAll()
	{
		Iterator iter = pool.iterator();
		while(iter.hasNext())
		{
			Thread handler = (Thread) iter.next();
			handler.start();
		}
	}

	public void stopAll()
	{
		Iterator iter = pool.iterator();
		while(iter.hasNext())
		{
			Thread handler = (Thread) iter.next();
			handler.interrupt();
			handler = null;
		}
	}	
	
	
	
	
	
	/**
	 * @param args
	 */
	public static void main(String[] args)
	{
		// TODO Auto-generated method stub

	}

}
