
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class ThreadPool
{

	private int	         max	    = 10;
	private int	         min	    = 2;
	private int	         current	= 0;

	private final Object	monitor	= new Object();
	private final List	 pool	    = new ArrayList();

	private Queue	     queue	    = null;


	public ThreadPool(Queue queue)
	{
		this(queue, 5, 10);
	}

	public ThreadPool(Queue queue, int min, int max)
	{
		this.queue = queue;
		this.min = min;
		this.max = max;
		init();
	}

	private void init()
	{
		for(int i = 0; i < min; i++)
		{
			pool.add(createThread(i));
		}
	}

	private synchronized Thread createThread(int index)
	{
		Thread thread = null;
		try
		{
			thread = new ReadWorker(queue, index);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		current++;
		return thread;
	}

	public void startAll()
	{
		synchronized (monitor)
		{
			Iterator iter = pool.iterator();
			while(iter.hasNext())
			{
				Thread thread = (Thread) iter.next();
				thread.start();
			}
		}
	}

	public void stopAll()
	{
		synchronized (monitor)
		{
			Iterator iter = pool.iterator();
			while(iter.hasNext())
			{
				Thread thread = (Thread) iter.next();
				thread.interrupt();
				thread = null;
			}
			pool.clear();
		}
	}

	public void addThread()
	{
		synchronized (monitor)
		{
			if(current < max)
			{
				Thread t = createThread(current);
				t.start();
				pool.add(t);
			}
		}
	}

	public void removeThread()
	{
		synchronized (monitor)
		{
			if(current > min)
			{
				Thread t = (Thread) pool.remove(0);
				t.interrupt();
				t = null;
			}
		}
	}

}
